* HiggsProd.F
* approximate calculation of the Higgs production cross-sections
* neutral-Higgs cross-section data are from Fabio Maltoni:
*   http://maltoni.home.cern.ch/maltoni/TeV4LHC
* charged-Higgs cross-section data are from Tilman Plehn
*   http://www.ph.ed.ac.uk/~tplehn/charged_higgs
* this file is part of FeynHiggs
* last modified 30 Jul 10 th

#include "externals.h"
#include "debug.h"
#include "FHCouplings.h"


	subroutine FHHiggsProd(error, sqrts, prodxs)
	implicit none
	integer error
	double precision sqrts, prodxs(nprodxs)

#include "FH.h"
#define __SUBROUTINE__ "FHHiggsProd"

	integer i, h
	character*8 s

	logical blurb
	character*2 higgs(4)
	data blurb /.TRUE./
	data higgs /"h0", "HH", "A0", "Hp"/

	do i = 1, nprodxs
	  prodxs(i) = -999
	enddo

	call CheckCouplings(error, iand(ieor(coup_valid, valid), 1))
	if( error .gt. 0 ) return

	if( blurb ) then
	  print *, "FHHiggsProd contains code by"
	  print *, "F. Maltoni et al. (SM Higgs production cross-sections)"
	  print *, "http://maltoni.home.cern.ch/maltoni/TeV4LHC"
	  print *, "T. Plehn et al. (charged Higgs production cross-sections)"
	  print *, "http://www.ph.ed.ac.uk/~tplehn/charged_higgs"
	  print *, "R. Harlander et al. (SM bbH production cross-sections)"
	  print *, "hep-ph/0304035"
	  print *, "M. Grazzini et al. (SM ggH prediction)"
	  print *, "http://theory.fi.infn.it/grazzini/hcalculators.html"
	  blurb = .FALSE.
	endif

	if( int(sqrts + .05D0) .eq. 2 ) then
	  call HiggsProdTev(error, prodxs)
	  s = "Tevatron"
	else if( sqrts .gt. 2.5D0 ) then
	  call HiggsProdLHC(error, sqrts, prodxs)
	  s = "LHC"
	else
	  Error("Bad energy")
	  return
	endif

	if( debuglevel .lt. 2 ) return

	DPROD s, " production cross-sections (MSSM, SM):" ENDL
	DPROD "sqrt(s) =", sqrts

	do h = 1, 3
	  DPROD "b-b-"//higgs(h)//"    =", bbh(h), bbhSM(h)	ENDL
	  DPROD "btag-b-"//higgs(h)//" =", btagbh(h), btagbhSM(h) ENDL
	  DPROD "g-g-"//higgs(h)//"    =", ggh(h), gghSM(h)	ENDL
	  DPROD "q-q-"//higgs(h)//"    =", qqh(h), qqhSM(h)	ENDL
	  DPROD "t-t-"//higgs(h)//"    =", tth(h), tthSM(h)	ENDL
	  DPROD "W-"//higgs(h)//"      =", Wh(h), WhSM(h)	ENDL
	  DPROD "Z-"//higgs(h)//"      =", Zh(h), ZhSM(h)	ENDL
	  DPROD "St-St-"//higgs(h)//"  =", StSth(h)		ENDL
	enddo
	DPROD "t-H-m     =", tHm ENDL
	end


************************************************************************

	subroutine HiggsProdTev(error, prodxs)
	implicit none
	integer error
	double precision prodxs(nprodxs)

#include "FH.h"

	integer h
	double precision m, sqrtm, logm, c, fac, sqme(2)
	double precision kTopNLO, kTopNNLO
	double precision kBotReNLO, kBotImNLO

	do h = 1, 3
	  if( MHiggs(h) .gt. 80 ) then
	    m = MHiggs(h)
	    sqrtm = sqrt(m)
	    logm = log(m)

#include "NHiggsProdFits-Tev.f"
#include "kfactor-Tev.f"

	    c = GammaSM(H0FF(h,4,3,3))
	    if( c .eq. 0 ) then
	      bbh(h) = 0
	      bbhSM(h) = 0
	      btagbh(h) = 0
	      btagbhSM(h) = 0
	    else
	      c = Gamma(H0FF(h,4,3,3))/c
	      bbh(h) = c*bbhSM(h)
	      btagbh(h) = c*btagbhSM(h)
	    endif

	    c = ratios(H0FF(h,3,3,3))
	    tth(h) = c*tthSM(h)

	    call SetDecayScale(error, m)
	    call Kin1to2(fac, h, 0D0, 0D0, .5D0)

	    call SetKFactors(0D0, kTopNLO, kBotReNLO, kBotImNLO)
	    call hggSM_SquaredME(sqme, O'155', -1)
	    c = sqme(2)
	    if( c .eq. 0 ) then
	      ggh(h) = 0
	    else
	      call SetKFactors(1D0, kTopNLO*kTopNNLO,
     &          kBotReNLO, kBotImNLO)
	      call hgg_SquaredME(sqme, O'155', -1)
	      ggh(h) = gghSM(h)*sqme(2)/c
	    endif

	    c = ratios(H0VV(h,4))
	    qqh(h) = c*qqhSM(h)
	    Wh(h) = c*WhSM(h)

	    c = ratios(H0VV(h,3))
	    Zh(h) = c*ZhSM(h)
	  endif
	enddo
	end


************************************************************************

	subroutine HiggsProdLHC(error, sqrts, prodxs)
	implicit none
	integer error
	double precision sqrts, prodxs(nprodxs)

#include "FH.h"

	integer h
	double precision logsqrts, m, sqrtm, logm
	double precision c, fac, sqme(2), TBeff
	double precision kTopNLO, kTopNNLO
	double precision kBotReNLO, kBotImNLO
	double precision gghSMNLO(3)

	logsqrts = log(sqrts)

	do h = 1, 3
	  if( MHiggs(h) .gt. 80 ) then
	    m = MHiggs(h)
	    sqrtm = sqrt(m)
	    logm = log(m)

#include "NHiggsProdFits-LHC.f"
#include "kfactor-LHC.f"

	    call SetDecayScale(error, m)
	    call Kin1to2(fac, h, 0D0, 0D0, .5D0)

	    call SetKFactors(0D0, kTopNLO, kBotReNLO, kBotImNLO)
	    call hggSM_SquaredME(sqme, O'155', -1)
	    c = sqme(2)
	    if( c .eq. 0 ) then
	      ggh(h) = 0
	    else
	      call SetKFactors(1D0, kTopNLO*kTopNNLO,
     &          kBotReNLO, kBotImNLO)
	      call hgg_SquaredME(sqme, O'155', -1)
	      ggh(h) = gghSMNLO(h)*sqme(2)/c
	    endif

	    c = GammaSM(H0FF(h,4,3,3))
	    if( c .eq. 0 ) then
	      bbh(h) = 0
	      bbhSM(h) = 0
	    else
	      c = Gamma(H0FF(h,4,3,3))/c
	      bbh(h) = c*bbhSM(h)
	    endif

	    if( abs(sqrts - 14) .gt. 1D-5 ) goto 99

#include "NHiggsProdFits-LHC14.f"

	    if( c .eq. 0 ) then
	      btagbh(h) = 0
	      btagbhSM(h) = 0
	    else
	      btagbh(h) = c*btagbhSM(h)
	    endif

	    c = ratios(H0FF(h,3,3,3))
	    tth(h) = c*tthSM(h)

	    c = ratios(H0VV(h,4))
	    qqh(h) = c*qqhSM(h)
	    Wh(h) = c*WhSM(h)

	    c = ratios(H0VV(h,3))
	    Zh(h) = c*ZhSM(h)

	    StSth(h) = StSth(h)*
     &        Sq(Coupling(H0SfSf(h,1,1,3,3)))

99	  continue
	  endif
	enddo

	if( sqrts .eq. 14 .and. MHiggs(4) .gt. 80 ) then
	  sqrtm = sqrt(MHiggs(4))
	  TBeff = TB/sqrt(abs(1 + Deltaf(4,3)))

#include "CHiggsProdFits-LHC14.f"

	  tHm = tHm/abs(1 + Deltaf(4,3))
	endif
	end


************************************************************************

	subroutine SetKFactors(k0, kt, kbre, kbim)
	implicit none
	double precision k0, kt, kbre, kbim

#include "FH.h"

	integer i

	do i = 1, 3*3
	  hggU_flat(i) = k0
	  hggDRe_flat(i) = k0
	  hggDIm_flat(i) = k0
	enddo
	hggU(1,3) = kt
	hggU(2,3) = kt
	hggU(3,3) = kt
	hggDRe(1,3) = kbre
	hggDRe(2,3) = kbre
	hggDRe(3,3) = kbre
	hggDIm(1,3) = kbim
	hggDIm(2,3) = kbim
	hggDIm(3,3) = kbim
	end

